Package: ReSurv
Type: Package
Title: Machine Learning Models for Predicting Claim Counts
Version: 1.0.0
Authors@R: 
    c(person(given = "Emil",
             family = "Hofman",
             role = c("aut", "cre", "cph"),
             email="emil_hofman@hotmail.dk"),
      person(given = "Gabriele",
             family = "Pittarello",
             role = c("aut", "cph"),
             email = "gabriele.pittarello@uniroma1.it",
             comment = c(ORCID = "0000-0003-3360-5826")),
      person(given = "Munir",
             family = "Hiabu",
             email="mh@math.ku.dk",
             role = c("aut", "cph"),
             comment = c(ORCID = "0000-0001-5846-667X")))
Description: Prediction of claim counts using the feature based development factors introduced in the manuscript Hiabu M., Hofman E. and Pittarello G. (2023) <doi:10.48550/arXiv.2312.14549>. 
             Implementation of Neural Networks, Extreme Gradient Boosting, 
             and Cox model with splines to optimise the partial log-likelihood of proportional hazard models.
URL: https://github.com/edhofman/ReSurv
BugReports: https://github.com/edhofman/ReSurv/issues
License: GPL (>= 2)
Depends: tidyverse
Imports: stats, dplyr, dtplyr, fastDummies, forecast, data.table,
        purrr, tidyr, tibble, ggplot2, survival, reshape2, bshazard,
        SynthETIC, rpart, reticulate, xgboost, SHAPforxgboost
SystemRequirements: Python (>= 3.8.0)
Encoding: UTF-8
Suggests: knitr, rmarkdown
VignetteBuilder: knitr, rmarkdown
RoxygenNote: 7.3.2
NeedsCompilation: no
Packaged: 2024-11-14 08:40:52 UTC; gpitt
Author: Emil Hofman [aut, cre, cph],
  Gabriele Pittarello [aut, cph]
    (<https://orcid.org/0000-0003-3360-5826>),
  Munir Hiabu [aut, cph] (<https://orcid.org/0000-0001-5846-667X>)
Maintainer: Emil Hofman <emil_hofman@hotmail.dk>
Repository: CRAN
Date/Publication: 2024-11-14 16:00:10 UTC
Built: R 4.6.0; ; 2026-05-01 06:12:24 UTC; unix
